import pychromecast
from prettytable import PrettyTable


def discover_chromecasts():
    # Fetch known devices
    pf = open("known_devices.txt", "w+")
    known_devices = pf.read().strip()
    pf.close()
    known_devices_array = []
    if known_devices:
        known_devices_array = known_devices.split(',')
        print('Previosly known chromecasts:')
        ip_table = PrettyTable()
        ip_table.field_names = ["ip"]
        for device in known_devices_array:
            ip_table.add_row([device])
        print(ip_table)
    else:
        print('No previously known chromecasts')
    # Discover devices
    print('Searching chromecasts...')
    if len(known_devices_array) > 0:
        services, browser = pychromecast.discovery.discover_chromecasts(known_hosts=known_devices_array)
    else:
        services, browser = pychromecast.discovery.discover_chromecasts()
    # Stop discovery
    pychromecast.discovery.stop_discovery(browser)
    # print chromecasts
    if services:
        ips = []
        chromecast_count = len(services)
        print('Found ' + str(chromecast_count) + ' chromecasts.')
        # print(services)
        table = PrettyTable()
        table.field_names = ["name", "ip", "port", "uuid", "type"]
        for chromecast in services:
            ip = chromecast.host
            port = chromecast.port
            uuid = str(chromecast.uuid)
            c_type = chromecast.model_name
            name = chromecast.friendly_name
            ips.append(ip)
            table.add_row([name, ip, port, uuid, c_type])

        f = open("known_devices.txt", "w")
        f.write(','.join(ips))
        f.close()
        print(table)
        return services
    else:
        print('No devices found')
        return None



from flask import Flask, Blueprint

from . import discover_devices, discover_media, player

bp = Blueprint('endpoints', __name__)

@bp.route("/discover")
def home():
    services = discover_devices.discover_chromecasts()
    chromecasts = []

    for chromecast in services:
        chromecasts.append({
            "ip": chromecast.host,
            "port": chromecast.port,
            "uuid": str(chromecast.uuid),
            "type": chromecast.model_name,
            "name": chromecast.friendly_name
        })

    return {"devices": chromecasts}

@bp.route("/oldcast/<ip>/<port>")
def oldcast(ip, port):
    player.play_to_chromecast(ip, port)
    return f"Success to {ip} and {port}??"

@bp.route("/cast/<input>/<ip>/<port>")
def cast(input, ip, port):
    player.play_to_chromecast2(input, ip, port)
    return f"Success to {ip} and {port}??"

@bp.route("/test")
def test():
    return {"inputs": discover_media.discover_inputs()}

@bp.route("/play")
def play():
    player.play()
    return "All good"

@bp.route("/stop")
def stop():
    player.stop()
    return "All good 1"


#import wmi


def discover_usb_line_in():
    print("Test")
    # c  = wmi.WMI()
    # 
    # result = []
    # for item in c.Win32_USBControllerDevice():
    #     print(item)
    #     result.append(item)
    # return result

def discover_inputs():
    import sounddevice as sd
    devices = sd.query_devices()
    input_devices = [device for device in devices if device['max_input_channels'] > 0]
    input_names = list(set(map(lambda d: d['name'], input_devices)))
    return input_names

def test():
    print("Test2")
    import os
    import win32api
    import win32file
    # os.system("cls")
    drive_types = {
                    win32file.DRIVE_UNKNOWN : "Unknown\nDrive type can't be determined.",
                    win32file.DRIVE_REMOVABLE : "Removable\nDrive has removable media. This includes all floppy drives and many other varieties of storage devices.",
                    win32file.DRIVE_FIXED : "Fixed\nDrive has fixed (nonremovable) media. This includes all hard drives, including hard drives that are removable.",
                    win32file.DRIVE_REMOTE : "Remote\nNetwork drives. This includes drives shared anywhere on a network.",
                    win32file.DRIVE_CDROM : "CDROM\nDrive is a CD-ROM. No distinction is made between read-only and read/write CD-ROM drives.",
                    win32file.DRIVE_RAMDISK : "RAMDisk\nDrive is a block of random access memory (RAM) on the local computer that behaves like a disk drive.",
                    win32file.DRIVE_NO_ROOT_DIR : "The root directory does not exist."
                }
    
    drives = win32api.GetLogicalDriveStrings().split('\x00')[:-1]

    for device in drives:
        type = win32file.GetDriveType(device)
        
        print("Drive: %s" % device)
        print(drive_types[type])
        print("-"*72)

    # os.system('pause')
    


import vlc

# Line in
# vlc --intf rc --no-video dshow:// vdev="none" adev="Line In (USB AUDIO CODEC)" --sout="#chromecast{ip=192.168.86.31, port=8009}" --demux-filter=demux_chromecast
# From file
# vlc --intf rc --no-video /Downloads/sample.mp3 --sout="#chromecast{ip=192.168.86.27, port=32089}" --demux-filter=demux_chromecast
_instance: vlc.Instance = None
player: vlc.MediaPlayer = None
mp3_file = "app/sample.mp3"

def play():
    m: vlc.Media = instance().media_new(mp3_file)
    stop()
    global player
    player = m.player_new_from_media()
    player.play()

def play_to_chromecast(chromecast_ip, port):
    m: vlc.Media = instance().media_new(mp3_file)
    m.add_option(f":sout=#chromecast{{ip={chromecast_ip}, port={port}}}")
    m.add_option(f":demux-filter=demux_chromecast")
    stop()
    global player
    player = m.player_new_from_media()
    player.play()

def play_to_chromecast2(input, chromecast_ip, port):
    m: vlc.Media = instance().media_new_location("dshow://")
    m.add_option(f":adev={input}")
    m.add_option(":dshow-vdev=None")
    m.add_option(f":sout=#chromecast{{ip={chromecast_ip}, port={port}}}")
    m.add_option(f":demux-filter=demux_chromecast")
    stop()
    global player
    player = m.player_new_from_media()
    player.play()
    
def test():
    print("This is a test")

def stop():
    global player
    if player:
        player.stop()
        player.release()
        player = None

def instance():
    global _instance
    if _instance is None:
        _instance = vlc.Instance()
    return _instance
